// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

using System.Collections;
using System;

namespace fit
{
	public abstract class RowFixture : ColumnFixture 
	{
		internal object[] results;
		internal ArrayList missing = new ArrayList();
		internal ArrayList surplus = new ArrayList();

		public override void doRows(Parse rows) 
		{
			try 
			{
				bind(rows.parts);
				results = query();
				match(list(rows.more), list(results), 0);
				Parse last = rows.last();
				last.more = buildRows(surplus.ToArray());
				mark(last.more, "surplus");
				mark(missing, "missing");
			} 
			catch (Exception e) 
			{
				exception (rows.leaf(), e);
			}
		}

		public abstract object[] query();  // get rows to be compared
		protected internal abstract override Type getTargetClass();             // get expected type of row

		void match(ArrayList expected, ArrayList computed, int col) 
		{
			Hashtable eMap = eSort(expected, col);
			Hashtable cMap = cSort(computed, col);
			foreach (object key in union(eMap.Keys, cMap.Keys))
			{
				ArrayList eList = (ArrayList)eMap[key];
				ArrayList cList = (ArrayList)cMap[key];
				if (eList == null) 
				{
					surplus.AddRange(cList);
				} 
				else if (cList == null) 
				{
					missing.AddRange(eList);
				} 
				else if (eList.Count==1 && cList.Count==1) 
				{
					check(eList, cList);
				} 
				else if (col+1 < columnBindings.Length) 
				{
					match(eList, cList, col+1);
				} 
				else 
				{
					check(eList, cList);
				}
			}
		}

		ArrayList list(Parse rows) 
		{
			ArrayList result = new ArrayList();
			while (rows != null) 
			{
				result.Add(rows);
				rows = rows.more;
			}
			return result;
		}

		ArrayList list(object[] rows) 
		{
			return new ArrayList(rows);
		}

		Hashtable eSort(ArrayList list, int col) 
		{
			TypeAdapter a = columnBindings[col];
			Hashtable result = new Hashtable();
			foreach (Parse row in list)
			{
				Parse cell = row.parts.at(col);
				try 
				{
					object key = a.parse(cell.text());
					bin(result, key, row);
				} 
				catch (Exception e) 
				{
					exception(cell, e);
				}
			}
			return result;
		}

		Hashtable cSort(ArrayList list, int col) 
		{
			TypeAdapter a = columnBindings[col];
			Hashtable result = new Hashtable();
			foreach (object row in list)
			{
				try 
				{
					a.target = row;
					object key = a.get();
					bin(result, key, row);
				} 
				catch (Exception) 
				{
					// surplus anything with bad keys, including null
					surplus.Add(row);
				}
			}
			return result;
		}

		void bin (Hashtable map, object key, object row) 
		{
			if (map.ContainsKey(key)) 
			{
				((ArrayList)map[key]).Add(row);
			} 
			else 
			{
				ArrayList list = new ArrayList();
				list.Add(row);
				map[key] = list;
			}
		}

		ICollection union(ICollection a, ICollection b) 
		{
			Hashtable result = new Hashtable();
			foreach (object o in a)
			{
				result[o] = true;
			}
			foreach (object o in b)
			{
				result[o] = true;
			}
			return result.Keys;
		}

		void check (ArrayList eList, ArrayList cList) 
		{
			if (eList.Count==0) 
			{
				surplus.AddRange(cList);
				return;
			}
			if (cList.Count==0) 
			{
				missing.AddRange(eList);
				return;
			}
			Parse row = (Parse)eList[0];
			eList.RemoveAt(0);
			Parse cell = row.parts;
			Object obj = cList[0];
			cList.RemoveAt(0);
			for (int i=0; i<columnBindings.Length; i++) 
			{
				TypeAdapter a = columnBindings[i];
				a.target = obj;
				check(cell, a);
				cell = cell.more;
			}
			check (eList, cList);
		}

		void mark(Parse rows, string message) 
		{
			string annotation = label(message);
			while (rows != null) 
			{
				wrong(rows.parts);
				rows.parts.addToBody(annotation);
				rows = rows.more;
			}
		}

		void mark(ArrayList rows, string message) 
		{
			string annotation = label(message);
			foreach (Parse row in rows)
			{
				wrong(row.parts);
				row.parts.addToBody(annotation);
			}
		}

		Parse buildRows(object[] rows) 
		{
			Parse root = new Parse(null ,null, null, null);
			Parse next = root;
			for (int i=0; i<rows.Length; i++) 
			{
				next = next.more = new Parse("tr", null, buildCells(rows[i]), null);
			}
			return root.more;
		}

		Parse buildCells(object row) 
		{
			if (row == null) 
			{
				Parse nil = new Parse("td", "null", null, null);
				nil.addToTag(" colspan="+columnBindings.Length);
				return nil;
			}
			Parse root = new Parse(null, null, null, null);
			Parse next = root;
			for (int i=0; i<columnBindings.Length; i++) 
			{
				next = next.more = new Parse("td", "&nbsp;", null, null);
				TypeAdapter a = columnBindings[i];
				if (a == null) 
				{
					ignore (next);
				} 
				else 
				{
					try 
					{
						a.target = row;
						next.body = a.get().ToString();
					} 
					catch (Exception e) 
					{
						exception(next, e);
					}
				}
			}
			return root.more;
		}
	}
}